package Test::Repocop::SQL;
use strict;
use warnings;
use DBI;
use File::Basename;
use Test::Repocop::Log;
use Test::Repocop::Common;
use Test::Repocop::TestDB;

# TODO: configure
my $sqlsh='/usr/bin/sqlite3';

my $sth_insert_files;
my $sth_insert_metadata;
my $sth_insert_triggers;

sub init_db {
    my ($dbfile, $sqlfile, $testname)=@_;
    my $schema_ver=0;
    $schema_ver=$1 if $sqlfile=~/\.(\d+)$/;
    if (-e $dbfile) {
	my $dbh=&connect_db($dbfile);
	my $sth=$dbh->prepare("PRAGMA user_version");
	$sth->execute();
	my @present_version=$sth->fetchrow_array();
	&disconnect_db($dbh);
	return if ($present_version[0]==$schema_ver);
	if ($present_version[0]>$schema_ver) {
	    die "repocop: error initializing db:\nThe test or collector '$testname' requires version $schema_ver of database $dbfile, but the db already has version $present_version[0].\nIt looks like you used to run next version of the test/collector previously.\nPlease, upgrade your test/collector or manually remove the database file.\n";
	} else {
	    my $testdir=dirname($sqlfile);
	    my $is_upgraded=0;
	    my @upgrade_sql= sort {$a=~/\.(\d+)$/; my $a1=$1;$b=~/\.(\d+)$/; my $b1=$1; $a1 <=> $b1} grep {m!/upgrade\.sql\.(\d+)$!&& $1>$present_version[0] && $1<=$schema_ver} glob "$testdir/upgrade.sql.*";
	    if (@upgrade_sql>0) {
		foreach my $upgrade_sql (@upgrade_sql) {
		    &repocop_note("running $upgrade_sql");
		    system ("sqlite3 $dbfile < $upgrade_sql")==0 or die "can't upgrade sqlite3 db $dbfile with $upgrade_sql: $?";
		}
		$is_upgraded=1;
	    }
	    if (-x "$testdir/upgrade") {
		my $testname=basename($testdir);
		&Test::Repocop::Common::set_test_environment($testname);
		&repocop_note("running $testdir/upgrade");
		system ("$testdir/upgrade",$present_version[0],$schema_ver)==0 or die "upgrade script $testdir/upgrade failed: $?";
		$is_upgraded=1;
	    }
	    unless ($is_upgraded) {
		# remove obsolete db file
		unlink $dbfile;
		&Test::Repocop::TestDB->new()->remove_test($testname);
		&repocop_note("Database $dbfile has version $present_version[0] while test requires version $schema_ver - database discarded.");
	    }
	}
    }
    unless (-e $dbfile) {
	# create empty db
	if (system ("sqlite3 $dbfile < $sqlfile")!=0) {
	    unlink $dbfile if -e $dbfile; # it creates empty useless db
	    die "can't initialize sqlite3 db $dbfile with $sqlfile: $?";
	}
    }
    my $dbh=&connect_db($dbfile);
    my $sth=$dbh->prepare("PRAGMA user_version");
    $sth->execute();
    my @present_version=$sth->fetchrow_array();
    # force schema version
    if ($present_version[0]!=$schema_ver) {
	$dbh->do("PRAGMA user_version=$schema_ver");
    }
    &disconnect_db($dbh);

}

sub connect_db {
    my ($dbfile)=@_;
    my $dbh = DBI->connect("dbi:SQLite:dbname=$dbfile","","", {
	PrintError => 1,
	AutoCommit => 0,
			});
    die "can't connect to $dbfile\n" unless $dbh;
    return $dbh;
}

sub disconnect_db {
    my ($dbh)=@_;
    return unless $dbh;
    $dbh->commit;
    # hack around closing dbh with active statement handles bug
    local $SIG{'__WARN__'} = sub {};
    $dbh->disconnect;
}

sub purge_db {
    my ($dbfile,$is_except,$pkgid_ref,$purge_sql)=@_;
    my $dbh = &connect_db($dbfile);
    my $sqlquery="SELECT name,sql from sqlite_master WHERE type='table'";
    my $sth=$dbh->prepare($sqlquery) or die $dbh->errstr;
    $sth->execute() or die $dbh->errstr;
    my $result=$sth->fetchall_arrayref;
    if ($result) {
	foreach my $tblrowref (@$result) {
	    if ($tblrowref->[1]=~/\s(?:PKGID|pkgid)\s/) {
		my $delsql="DELETE FROM $tblrowref->[0] WHERE PKGID ".($is_except ? 'NOT ':'')."IN ('".join("','", @$pkgid_ref)."')";
		&repocop_debug('purging ',$dbfile,': executing ',$delsql);
		$dbh->do($delsql) or die $dbh->errstr;
	    }
	}
    }
    $dbh->commit or die $dbh->errstr;
    &disconnect_db($dbh);
    system ("sqlite3 $dbfile < $purge_sql")==0 or die "can't purge sqlite3 db $dbfile with $purge_sql: $?" if $purge_sql;
}

sub vacuum_db {
    my ($dbfile)=@_;
    system("echo 'VACUUM;' | sqlite3 $dbfile")==0 or die "can't vacuum sqlite3 db $dbfile: $?";;
}

1;

=head1	NAME

Test::Repocop::SQL -   for repocop

=head1	DESCRIPTION

TODO

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop.

=head1	COPYING

Copyright (c) 2008-2022 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

